@@ -96,7 +96,7 @@ int setupDatagramSocket(UsageEnvironment& env, Port port,
   // Note: Windoze requires binding, even if the port number is 0
 #if defined(__WIN32__) || defined(_WIN32)
 #else
-  if (port.num() != 0) {
+  if (port.num() != 0 || ReceivingInterfaceAddr != INADDR_ANY) {
 #endif
     struct sockaddr_in name;
     name.sin_family = AF_INET;
@@ -173,7 +173,7 @@ int setupStreamSocket(UsageEnvironment& env,
   // Note: Windoze requires binding, even if the port number is 0
 #if defined(__WIN32__) || defined(_WIN32)
 #else
-  if (port.num() != 0) {
+  if (port.num() != 0 || ReceivingInterfaceAddr != INADDR_ANY) {
 #endif
     struct sockaddr_in name;
     name.sin_family = AF_INET;
@@ -657,12 +657,7 @@ netAddressBits ourSourceAddressForMulticast(UsageEnvironment& env) {
 	// Use our newly-discovered IP address, and the current time,
 	// to initialize the random number generator's seed:
 	struct timeval timeNow;
-#ifdef BSD
-	struct timezone Idunno;
-#else
-	int Idunno;
-#endif
-	gettimeofday(&timeNow, &Idunno);
+	gettimeofday(&timeNow, NULL);
 	unsigned seed = ourAddress^timeNow.tv_sec^timeNow.tv_usec;
 	our_srandom(seed);
 
@@ -683,12 +678,7 @@ netAddressBits chooseRandomIPv4SSMAddress(UsageEnvironment& env) {
 
 char const* timestampString() {
 	struct timeval tvNow;
-#ifdef BSD
-	struct timezone Idunno;
-#else
-	int Idunno;
-#endif
-	gettimeofday(&tvNow, &Idunno);
+	gettimeofday(&tvNow, NULL);
 
 	static char timeString[9]; // holds hh:mm:ss plus trailing '\0'
 	char const* ctimeResult = ctime((time_t*)&tvNow.tv_sec);
