@@ -19,6 +19,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 // Implementation
 
 #include "BasicUDPSource.hh"
+#include <GroupsockHelper.hh>
 
 BasicUDPSource* BasicUDPSource::createNew(UsageEnvironment& env,
 					Groupsock* inputGS) {
@@ -27,6 +28,8 @@ BasicUDPSource* BasicUDPSource::createNew(UsageEnvironment& env,
 
 BasicUDPSource::BasicUDPSource(UsageEnvironment& env, Groupsock* inputGS)
   : FramedSource(env), fInputGS(inputGS) {
+  // Try to use a large receive buffer (in the OS):
+  increaseReceiveBufferTo(env, inputGS->socketNum(), 50*1024);
 }
 
 BasicUDPSource::~BasicUDPSource(){
