@@ -34,7 +34,7 @@ MP3FileSource::~MP3FileSource() {
 }
 
 char const* MP3FileSource::MIMEtype() const {
-  return "audio/mpeg";
+  return "audio/MPEG";
 }
 
 MP3FileSource* MP3FileSource::createNew(UsageEnvironment& env, char const* fileName) {
@@ -64,6 +64,10 @@ float MP3FileSource::filePlayTime() const {
   return fStreamState->filePlayTime();
 }
 
+void MP3FileSource::setPresentationTimeScale(unsigned scale) {
+  fStreamState->setPresentationTimeScale(scale);
+}
+
 void MP3FileSource::seekWithinFile(float seekNPT) {
   fStreamState->seekWithinFile(seekNPT);
 }
