@@ -110,7 +110,7 @@ MPEG1or2Demux* MPEG1or2Demux
 }
 
 MPEG1or2Demux::SCR::SCR()
-  : highBit(0), remainingBits(0), extension(0) {
+  : highBit(0), remainingBits(0), extension(0), isValid(False) {
 }
 
 void MPEG1or2Demux
@@ -120,6 +120,10 @@ void MPEG1or2Demux
   }
 }
 
+void MPEG1or2Demux::flushInput() {
+  fParser->flushInput();
+}
+
 MPEG1or2DemuxedElementaryStream*
 MPEG1or2Demux::newElementaryStream(u_int8_t streamIdTag) {
   ++fNumOutstandingESs;
@@ -312,6 +316,7 @@ void MPEG1or2Demux::handleClosure(void* clientData) {
   }
 }
 
+
 ////////// MPEGProgramStreamParser implementation //////////
 
 #include <string.h>
@@ -418,6 +423,7 @@ void MPEGProgramStreamParser::parsePackHeader() {
     scr.remainingBits |= (next4Bytes&0xFFFE0000)>>2;
     scr.remainingBits |= (next4Bytes&0x0000FFFE)>>1;
     scr.extension = 0;
+    scr.isValid = True;
     skipBits(24);
 
 #if defined(DEBUG_TIMESTAMPS) || defined(DEBUG_SCR_TIMESTAMPS)
@@ -436,6 +442,7 @@ void MPEGProgramStreamParser::parsePackHeader() {
     scr.extension = (next4Bytes&0x00000003)<<7;
     next4Bytes = get4Bytes();
     scr.extension |= (next4Bytes&0xFE000000)>>25;
+    scr.isValid = True;
     skipBits(5);
 
 #if defined(DEBUG_TIMESTAMPS) || defined(DEBUG_SCR_TIMESTAMPS)
