@@ -36,6 +36,7 @@ MPEG2TransportStreamFromPESSource
 				    MPEG1or2DemuxedElementaryStream* inputSource)
   : FramedFilter(env, inputSource),
     fOutgoingPacketCounter(0), fProgramMapVersion(0),
+    fPreviousInputProgramMapVersion(0xFF), fCurrentInputProgramMapVersion(0xFF),
     fPCR_PID(0), fCurrentPID(0),
     fPCRHighBit(0), fPCRRemainingBits(0), fPCRExtension(0),
     fInputBufferSize(0), fInputBufferBytesUsed(0) {
@@ -43,6 +44,7 @@ MPEG2TransportStreamFromPESSource
     fPIDState[i].counter = 0;
     fPIDState[i].streamType = 0;
   }
+
   fInputBuffer = new unsigned char[MAX_PES_PACKET_SIZE];
 }
 
@@ -68,9 +70,13 @@ void MPEG2TransportStreamFromPESSource::doGetNextFrame() {
     }
 
     // Periodically (or when we see a new PID) return a Program Map Table instead:
-    Boolean programMapHasChanged = fPIDState[fCurrentPID].counter == 0;
+    Boolean programMapHasChanged = fPIDState[fCurrentPID].counter == 0
+      || fCurrentInputProgramMapVersion != fPreviousInputProgramMapVersion;
     if (fOutgoingPacketCounter % PMT_FREQUENCY == 0 || programMapHasChanged) {
-      if (programMapHasChanged) fPIDState[fCurrentPID].counter = 1; // for next time
+      if (programMapHasChanged) { // reset values for next time:
+	fPIDState[fCurrentPID].counter = 1;
+	fPreviousInputProgramMapVersion = fCurrentInputProgramMapVersion;
+      }
       deliverPMTPacket(programMapHasChanged);
       break;
     }
@@ -105,23 +111,31 @@ void MPEG2TransportStreamFromPESSource
     u_int8_t stream_id = fInputBuffer[3];
     // Use this stream_id directly as our PID.
     // Also, figure out the Program Map 'stream type' from this.
-    // However, ignore "padding_stream" data
-    if (stream_id == 0xBE) {
+    if (stream_id == 0xBE) { // padding_stream; ignore
+      frameSize = 0;
+    } else if (stream_id == 0xBC) { // program_stream_map
+      setProgramStreamMap(frameSize);
       frameSize = 0;
     } else {
       fCurrentPID = stream_id;
       if (fPCR_PID == 0) fPCR_PID = fCurrentPID; // use this stream's SCR for PCR
 
-      // Set the stream's type based on whether it's audio or video, and MPEG 1 or 2:
       MPEG1or2DemuxedElementaryStream* source
 	= (MPEG1or2DemuxedElementaryStream*)fInputSource;
+
+      // Set the stream's type:
       u_int8_t& streamType = fPIDState[fCurrentPID].streamType; // alias
-      if ((stream_id&0xE0) == 0xC0) { // audio
-	streamType = source->mpegVersion() == 1 ? 3 : 4;
-      } else if ((stream_id&0xF0) == 0xE0) { // video
-	streamType = source->mpegVersion() == 1 ? 1 : 2;
-      } else { // something else, e.g., AC-3 uses private_stream1 (0xBD)
-	streamType = 0x81; // private 
+
+      if (streamType == 0) {
+	// Instead, set the stream's type to default values, based on whether
+	// the stream is audio or video, and whether it's MPEG-1 or MPEG-2:
+	if ((stream_id&0xE0) == 0xC0) { // audio
+	  streamType = source->mpegVersion() == 1 ? 3 : 4;
+	} else if ((stream_id&0xF0) == 0xE0) { // video
+	  streamType = source->mpegVersion() == 1 ? 1 : 2;
+	} else { // something else, e.g., AC-3 uses private_stream1 (0xBD)
+	  streamType = 0x81; // private 
+	}
       }
 
       if (fCurrentPID == fPCR_PID) {
@@ -315,6 +329,42 @@ void MPEG2TransportStreamFromPESSource::deliverPMTPacket(Boolean hasChanged) {
   delete[] pmtBuffer;
 }
 
+void MPEG2TransportStreamFromPESSource::setProgramStreamMap(unsigned frameSize) {
+  if (frameSize <= 16) return; // program_stream_map is too small to be useful
+  if (frameSize > 0xFF) return; // program_stream_map is too large
+
+  u_int16_t program_stream_map_length = (fInputBuffer[4]<<8) | fInputBuffer[5];
+  if ((u_int16_t)frameSize > 6+program_stream_map_length) {
+    frameSize = 6+program_stream_map_length;
+  }
+  
+  u_int8_t versionByte = fInputBuffer[6];
+  if ((versionByte&0x80) == 0) return; // "current_next_indicator" is not set
+  fCurrentInputProgramMapVersion = versionByte&0x1F;
+
+  u_int16_t program_stream_info_length = (fInputBuffer[8]<<8) | fInputBuffer[9];
+  unsigned offset = 10 + program_stream_info_length; // skip over 'descriptors'
+
+  u_int16_t elementary_stream_map_length
+    = (fInputBuffer[offset]<<8) | fInputBuffer[offset+1];
+  offset += 2;
+  frameSize -= 4; // sizeof CRC_32
+  if (frameSize > offset + elementary_stream_map_length) {
+    frameSize = offset + elementary_stream_map_length;
+  } 
+
+  while (offset + 4 <= frameSize) {
+    u_int8_t stream_type = fInputBuffer[offset];
+    u_int8_t elementary_stream_id = fInputBuffer[offset+1];
+
+    fPIDState[elementary_stream_id].streamType = stream_type;
+
+    u_int16_t elementary_stream_info_length
+      = (fInputBuffer[offset+2]<<8) | fInputBuffer[offset+3];
+    offset += 4 + elementary_stream_info_length;
+  }
+}
+
 static u_int32_t CRC32[256] = {
   0x00000000, 0x04c11db7, 0x09823b6e, 0x0d4326d9,
   0x130476dc, 0x17c56b6b, 0x1a864db2, 0x1e475005,
