@@ -47,6 +47,7 @@ public:
   virtual ~MPEG4VideoStreamParser();
 
 private: // redefined virtual functions:
+  virtual void flushInput();
   virtual unsigned parse();
 
 private:
@@ -110,10 +111,11 @@ MPEG4VideoStreamFramer::MPEG4VideoStreamFramer(UsageEnvironment& env,
 }
 
 MPEG4VideoStreamFramer::~MPEG4VideoStreamFramer() {
+  delete[] fConfigBytes; delete[] fNewConfigBytes;
 }
 
 void MPEG4VideoStreamFramer::startNewConfig() {
-  delete fNewConfigBytes; fNewConfigBytes = NULL;
+  delete[] fNewConfigBytes; fNewConfigBytes = NULL;
   fNumNewConfigBytes = 0;
 }
 
@@ -127,7 +129,7 @@ void MPEG4VideoStreamFramer
   memmove(configNew, fNewConfigBytes, fNumNewConfigBytes);
   memmove(&configNew[fNumNewConfigBytes], newConfigBytes, numNewBytes);
   
-  delete fNewConfigBytes; fNewConfigBytes = configNew;
+  delete[] fNewConfigBytes; fNewConfigBytes = configNew;
   fNumNewConfigBytes += numNewBytes;
 }
 
@@ -163,6 +165,19 @@ void MPEG4VideoStreamParser::setParseState(MPEGParseState parseState) {
   MPEGVideoStreamParser::setParseState();
 }
 
+void MPEG4VideoStreamParser::flushInput() {
+  fSecondsSinceLastTimeCode = 0;
+  fTotalTicksSinceLastTimeCode = 0;
+  fPrevNewTotalTicks = 0;
+  fPrevPictureCountDelta = 1;
+
+  StreamParser::flushInput();
+  if (fCurrentParseState != PARSING_VISUAL_OBJECT_SEQUENCE) {
+    setParseState(PARSING_VISUAL_OBJECT_SEQUENCE); // later, change to GOV or VOP? #####
+  }
+}
+
+
 unsigned MPEG4VideoStreamParser::parse() {
   try {
     switch (fCurrentParseState) {
