@@ -61,7 +61,7 @@ Boolean MediaSession::lookupByName(UsageEnvironment& env,
 MediaSession::MediaSession(UsageEnvironment& env)
   : Medium(env),
     fSubsessionsHead(NULL), fSubsessionsTail(NULL),
-    fConnectionEndpointName(NULL), fMaxPlayEndTime(0.0) {
+    fConnectionEndpointName(NULL), fMaxPlayEndTime(0.0f), fScale(1.0f) {
 #ifdef SUPPORT_REAL_RTSP
   RealInitSDPAttributes(this);
 #endif
@@ -437,12 +437,15 @@ Boolean MediaSession
       continue;
     }
 
-    // Try to create a source for this subsession:
-    if (!subsession->initiate(useSpecialRTPoffset)) return False;
+    Boolean wasAlreadyInitiated = subsession->readSource() != NULL;
+    if (!wasAlreadyInitiated) {
+      // Try to create a source for this subsession:
+      if (!subsession->initiate(useSpecialRTPoffset)) return False;
+    }
 
     // Make sure the source's MIME type is one that we handle:
     if (strcmp(subsession->readSource()->MIMEtype(), mimeType) != 0) {
-      subsession->deInitiate();
+      if (!wasAlreadyInitiated) subsession->deInitiate();
       continue;
     }
 
@@ -528,7 +531,7 @@ MediaSubsession::MediaSubsession(MediaSession& parent)
     fConfig(NULL), fMode(NULL),
     fPlayEndTime(0.0),
     fMCT_SLAP_SessionId(0), fMCT_SLAP_Stagger(0),
-    fVideoWidth(0), fVideoHeight(0), fVideoFPS(0), fNumChannels(1),
+    fVideoWidth(0), fVideoHeight(0), fVideoFPS(0), fNumChannels(1), fScale(1.0f),
     fRTPSocket(NULL), fRTCPSocket(NULL),
     fRTPSource(NULL), fRTCPInstance(NULL), fReadSource(NULL) {
 #ifdef SUPPORT_REAL_RTSP
@@ -629,6 +632,11 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
       // A UDP-packetized stream (*not* a RTP stream)
       fReadSource = BasicUDPSource::createNew(env(), fRTPSocket);
       fRTPSource = NULL; // Note!
+
+      if (strcmp(fCodecName, "MP2T") == 0) { // MPEG-2 Transport Stream
+	fReadSource = MPEG2TransportStreamFramer::createNew(env(), fReadSource);
+	    // this sets "durationInMicroseconds" correctly, based on the PCR values
+      }
     } else {
       // Check "fCodecName" against the set of codecs that we support,
       // and create our RTP source accordingly
@@ -679,7 +687,8 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
 	// (one 'ADU'ized MP3 frame per packet; no headers)
 	fRTPSource
 	  = SimpleRTPSource::createNew(env(), fRTPSocket, fRTPPayloadFormat,
-				       90000, "audio/mpa-robust" /*hack*/);
+				       fRTPTimestampFrequency,
+				       "audio/MPA-ROBUST" /*hack*/);
 	if (fRTPSource == NULL) break;
 	
 	// Add a filter that converts these ADUs to MP3 frames:
@@ -713,6 +722,12 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
 	  = MPEG1or2VideoRTPSource::createNew(env(), fRTPSocket,
 					      fRTPPayloadFormat,
 					      fRTPTimestampFrequency);
+      } else if (strcmp(fCodecName, "MP2T") == 0) { // MPEG-2 Transport Stream
+	fRTPSource = SimpleRTPSource::createNew(env(), fRTPSocket, fRTPPayloadFormat,
+						fRTPTimestampFrequency, "video/MP2T",
+						0, False);
+	fReadSource = MPEG2TransportStreamFramer::createNew(env(), fRTPSource);
+	    // this sets "durationInMicroseconds" correctly, based on the PCR values
       } else if (strcmp(fCodecName, "H261") == 0) { // H.261
 	fReadSource = fRTPSource
 	  = H261VideoRTPSource::createNew(env(), fRTPSocket,
@@ -761,7 +776,6 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
 		   || strcmp(fCodecName, "PCMA") == 0 // PCM a-law audio
 		   || strcmp(fCodecName, "L16") == 0 // 16-bit linear audio
 		   || strcmp(fCodecName, "MP1S") == 0 // MPEG-1 System Stream
-		   || strcmp(fCodecName, "MP2T") == 0 // MPEG-2 Transport Str
 		   || strcmp(fCodecName, "MP2P") == 0 // MPEG-2 Program Stream
 		   || strcmp(fCodecName, "L8") == 0 // 8-bit linear audio
 		   || strcmp(fCodecName, "SPEEX") == 0 // SPEEX audio
@@ -1028,6 +1042,22 @@ Boolean MediaSubsession::parseSDPAttribute_fmtp(char const* sdpLine) {
 	delete[] fConfig; fConfig = strDup(valueStr);
       } else if (sscanf(line, " mode = %[^; \t\r\n]", valueStr) == 1) {
 	delete[] fMode; fMode = strDup(valueStr);
+      } else {
+	// Some of the above parameters are Boolean.  Check whether the parameter
+	// names appear alone, without a "= 1" at the end:
+	if (sscanf(line, " %[^; \t\r\n]", valueStr) == 1) {
+	  if (strcmp(valueStr, "octet-align") == 0) {
+	    fOctetalign = 1;
+	  } else if (strcmp(valueStr, "cpresent") == 0) {
+            fCpresent = True;
+	  } else if (strcmp(valueStr, "crc") == 0) {
+	    fCRC = 1;
+	  } else if (strcmp(valueStr, "robust-sorting") == 0) {
+	    fRobustsorting = 1;
+	  } else if (strcmp(valueStr, "randomaccessindication") == 0) {
+	    fRandomaccessindication = True;
+	  }
+	}
       }
       delete[] valueStr;
 
