@@ -187,12 +187,6 @@ void MultiFramedRTPSource
   fReorderingBuffer->setThresholdTime(uSeconds);
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 #define ADVANCE(n) do { bPacket->skip(n); } while (0)
 
 void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
@@ -265,7 +259,7 @@ void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
   
     // Fill in the rest of the packet descriptor, and store it:
     struct timeval timeNow;
-    gettimeofday(&timeNow, &Idunno);
+    gettimeofday(&timeNow, NULL);
     bPacket->assignMiscParams(rtpSeqNo, rtpTimestamp, presentationTime,
 			      hasBeenSyncedUsingRTCP, rtpMarkerBit,
 			      timeNow);
@@ -300,6 +294,7 @@ void BufferedPacket::reset() {
   fUseCount = 0;
 }
 
+// The following function has been deprecated:
 unsigned BufferedPacket
 ::nextEnclosedFrameSize(unsigned char*& /*framePtr*/, unsigned dataSize) {
   // By default, use the entire buffered data, even though it may consist
@@ -308,6 +303,21 @@ unsigned BufferedPacket
   return dataSize;
 }
 
+void BufferedPacket
+::getNextEnclosedFrameParameters(unsigned char*& framePtr, unsigned dataSize,
+				 unsigned& frameSize,
+				 unsigned& frameDurationInMicroseconds) {
+  // By default, use the entire buffered data, even though it may consist
+  // of more than one frame, on the assumption that the client doesn't
+  // care.  (This is more efficient than delivering a frame at a time)
+
+  // For backwards-compatibilty with existing uses of (the now deprecated)
+  // "nextEnclosedFrameSize()", call that function to implement this one:
+  frameSize = nextEnclosedFrameSize(framePtr, dataSize);
+
+  frameDurationInMicroseconds = 0; // by default.  Subclasses should correct this.  
+}
+
 Boolean BufferedPacket::fillInData(RTPInterface& rtpInterface) {
   reset();
 
@@ -357,8 +367,10 @@ void BufferedPacket::use(unsigned char* to, unsigned toSize,
 			 Boolean& hasBeenSyncedUsingRTCP,
 			 Boolean& rtpMarkerBit) {
   unsigned char* origFramePtr = &fBuf[fHead];
-  unsigned char* newFramePtr = origFramePtr; //may change in the call below
-  unsigned frameSize = nextEnclosedFrameSize(newFramePtr, fTail - fHead);
+  unsigned char* newFramePtr = origFramePtr; // may change in the call below
+  unsigned frameSize, frameDurationInMicroseconds;
+  getNextEnclosedFrameParameters(newFramePtr, fTail - fHead,
+				 frameSize, frameDurationInMicroseconds);
   if (frameSize > toSize) {
     bytesTruncated = frameSize - toSize;
     bytesUsed = toSize;
@@ -376,6 +388,13 @@ void BufferedPacket::use(unsigned char* to, unsigned toSize,
   presentationTime = fPresentationTime;
   hasBeenSyncedUsingRTCP = fHasBeenSyncedUsingRTCP;
   rtpMarkerBit = fRTPMarkerBit;
+
+  // Update "fPresentationTime" for the next enclosed frame (if any):
+  fPresentationTime.tv_usec += frameDurationInMicroseconds;
+  if (fPresentationTime.tv_usec >= 1000000) {
+    fPresentationTime.tv_sec += fPresentationTime.tv_usec/1000000;
+    fPresentationTime.tv_usec = fPresentationTime.tv_usec%1000000;
+  }
 }
 
 BufferedPacket* BufferedPacketFactory
@@ -483,7 +502,7 @@ BufferedPacket* ReorderingPacketBuffer
   // our time threshold has been exceeded, then forget it, and return
   // the head packet instead:
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   unsigned uSecondsSinceReceived
     = (timeNow.tv_sec - fHeadPacket->timeReceived().tv_sec)*1000000
     + (timeNow.tv_usec - fHeadPacket->timeReceived().tv_usec);
