@@ -18,14 +18,10 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 // A sink that generates a QuickTime file from a composite media session
 // Implementation
 
-#if (defined(__WIN32__) || defined(_WIN32)) && !defined(_WIN32_WCE)
-#include <io.h>
-#include <fcntl.h>
-#endif
-
 #include "QuickTimeFileSink.hh"
 #include "QuickTimeGenericRTPSource.hh"
 #include "GroupsockHelper.hh"
+#include "OutputFile.hh"
 #include "H263plusVideoRTPSource.hh" // for the special header
 #include "MPEG4GenericRTPSource.hh" //for "samplingFrequencyFromAudioSpecificConfig()"
 #include "MPEG4LATMAudioRTPSource.hh" // for "parseGeneralConfigStr()"
@@ -206,12 +202,6 @@ private:
 
 ////////// QuickTimeFileSink implementation //////////
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 QuickTimeFileSink::QuickTimeFileSink(UsageEnvironment& env,
 				     MediaSession& inputSession,
 				     FILE* outFid,
@@ -292,7 +282,7 @@ QuickTimeFileSink::QuickTimeFileSink(UsageEnvironment& env,
   // Use the current time as the file's creation and modification
   // time.  Use Apple's time format: seconds since January 1, 1904
 
-  gettimeofday(&fStartTime, &Idunno);
+  gettimeofday(&fStartTime, NULL);
   fAppleCreationTime = fStartTime.tv_sec - 0x83dac000;
 
   // Begin by writing a "mdat" atom at the start of the file.
@@ -318,9 +308,6 @@ QuickTimeFileSink::~QuickTimeFileSink() {
   }
 }
 
-static FILE* openFileByName(UsageEnvironment& env, char const* fileName);
-    // forward
-
 QuickTimeFileSink*
 QuickTimeFileSink::createNew(UsageEnvironment& env,
 			     MediaSession& inputSession,
@@ -336,7 +323,7 @@ QuickTimeFileSink::createNew(UsageEnvironment& env,
   QuickTimeFileSink* newSink = NULL;
 
   do {
-    FILE* fid = openFileByName(env, outputFileName);
+    FILE* fid = OpenOutputFile(env, outputFileName);
     if (fid == NULL) break;
 
     return new QuickTimeFileSink(env, inputSession, fid, bufferSize,
@@ -439,7 +426,7 @@ void QuickTimeFileSink::onRTCPBye(void* clientData) {
   SubsessionIOState* ioState = (SubsessionIOState*)clientData;
 
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   unsigned secsDiff
     = timeNow.tv_sec - ioState->fOurSink.fStartTime.tv_sec;
 
@@ -510,33 +497,6 @@ void QuickTimeFileSink::completeOutputFile() {
   fHaveCompletedOutputFile = True;
 }
 
-// NOTE: The following is the same code that's in "FileSink"
-// Someday we should share it #####
-static FILE* openFileByName(UsageEnvironment& env, char const* fileName) {
-  FILE* fid;
-    
-  // Check for special case 'file names': "stdout" and "stderr"
-  if (strcmp(fileName, "stdout") == 0) {
-    fid = stdout;
-#if defined(__WIN32__) || defined(_WIN32)
-    _setmode(_fileno(stdout), _O_BINARY);       // convert to binary mode
-#endif
-  } else if (strcmp(fileName, "stderr") == 0) {
-    fid = stderr;
-#if defined(__WIN32__) || defined(_WIN32)
-    _setmode(_fileno(stderr), _O_BINARY);       // convert to binary mode
-#endif
-  } else {
-    fid = fopen(fileName, "wb");
-  }
-
-  if (fid == NULL) {
-    env.setResultMsg("unable to open file \"", fileName, "\"");
-  }
-
-  return fid;
-}
-
 
 ////////// SubsessionIOState, ChunkDescriptor implementation ///////////
 
@@ -684,9 +644,7 @@ void SubsessionIOState::setFinalQTstate() {
   ChunkDescriptor* chunk = fHeadChunk;
   while (chunk != NULL) {
     unsigned const numFrames = chunk->fNumFrames;
-    unsigned const dur
-      = numFrames*chunk->fFrameDuration/fOurSubsession.numChannels();
-
+    unsigned const dur = numFrames*chunk->fFrameDuration;
     fQTDurationT += dur;
 
     chunk = chunk->fNextChunk;
@@ -826,7 +784,7 @@ void SubsessionIOState::useFrame(SubsessionBuffer& buffer) {
   }
 
   // Write the data into the file:
-  fwrite(frameSource, frameSize, 1, fOurSink.fOutFid);
+  fwrite(frameSource, 1, frameSize, fOurSink.fOutFid);
 
   // If we have a hint track, then write to it also:
   if (hasHintTrack()) {
@@ -1266,10 +1224,11 @@ addAtom(moov);
   //  doesn't display the movie correctly unless the audio track
   //  (if present) appears before the video track.  So ensure this here.)
   MediaSubsessionIterator iter(fInputSession);
-  while ((fCurrentSubsession = iter.next()) != NULL) {
-    fCurrentIOState = (SubsessionIOState*)(fCurrentSubsession->miscPtr); 
+  MediaSubsession* subsession;
+  while ((subsession = iter.next()) != NULL) {
+    fCurrentIOState = (SubsessionIOState*)(subsession->miscPtr); 
     if (fCurrentIOState == NULL) continue;
-    if (strcmp(fCurrentSubsession->mediumName(), "audio") != 0) continue;
+    if (strcmp(subsession->mediumName(), "audio") != 0) continue;
 
     size += addAtom_trak();
 
@@ -1280,10 +1239,10 @@ addAtom(moov);
     }
   }
   iter.reset();
-  while ((fCurrentSubsession = iter.next()) != NULL) {
-    fCurrentIOState = (SubsessionIOState*)(fCurrentSubsession->miscPtr); 
+  while ((subsession = iter.next()) != NULL) {
+    fCurrentIOState = (SubsessionIOState*)(subsession->miscPtr); 
     if (fCurrentIOState == NULL) continue;
-    if (strcmp(fCurrentSubsession->mediumName(), "audio") == 0) continue;
+    if (strcmp(subsession->mediumName(), "audio") == 0) continue;
 
     size += addAtom_trak();
 
