@@ -24,16 +24,11 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 ////////// RTCPMemberDatabase //////////
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 class RTCPMemberDatabase {
 public:
-  RTCPMemberDatabase()
-    : fNumMembers(1 /*ourself*/), fTable(HashTable::create(ONE_WORD_HASH_KEYS)) {
+  RTCPMemberDatabase(RTCPInstance& ourRTCPInstance)
+    : fOurRTCPInstance(ourRTCPInstance), fNumMembers(1 /*ourself*/),
+      fTable(HashTable::create(ONE_WORD_HASH_KEYS)) {
   }
 
   virtual ~RTCPMemberDatabase() {
@@ -72,6 +67,7 @@ public:
   void reapOldMembers(unsigned threshold);
 
 private:
+  RTCPInstance& fOurRTCPInstance;
   unsigned fNumMembers;
   HashTable* fTable;
 };
@@ -103,7 +99,7 @@ void RTCPMemberDatabase::reapOldMembers(unsigned threshold) {
 #ifdef DEBUG
         fprintf(stderr, "reap: removing SSRC 0x%x\n", oldSSRC);
 #endif
-      remove(oldSSRC);
+      fOurRTCPInstance.removeSSRC(oldSSRC);
     }
   } while (foundOldMember);
 }
@@ -113,7 +109,7 @@ void RTCPMemberDatabase::reapOldMembers(unsigned threshold) {
 
 static double dTimeNow() {
     struct timeval timeNow;
-    gettimeofday(&timeNow, &Idunno);
+    gettimeofday(&timeNow, NULL);
     return (double) (timeNow.tv_sec + timeNow.tv_usec/1000000.0);
 }
 
@@ -142,7 +138,7 @@ RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
   double timeNow = dTimeNow();
   fPrevReportTime = fNextReportTime = timeNow;
 
-  fKnownMembers = new RTCPMemberDatabase;
+  fKnownMembers = new RTCPMemberDatabase(*this);
   fInBuf = new unsigned char[maxPacketSize];
   if (fKnownMembers == NULL || fInBuf == NULL) return;
 
@@ -375,7 +371,8 @@ void RTCPInstance::incomingReportHandler1() {
                 unsigned jitter = ntohl(*(unsigned*)pkt); ADVANCE(4);
                 unsigned timeLastSR = ntohl(*(unsigned*)pkt); ADVANCE(4);
                 unsigned timeSinceLastSR = ntohl(*(unsigned*)pkt); ADVANCE(4);
-                transmissionStats.noteIncomingRR(reportSenderSSRC, lossStats,
+                transmissionStats.noteIncomingRR(reportSenderSSRC, fromAddress,
+						 lossStats,
 						 highestReceived, jitter,
 						 timeLastSR, timeSinceLastSR);
               } else {
@@ -537,8 +534,16 @@ int RTCPInstance::checkNewSSRC() {
 				       fOutgoingReportCount);
 }
 
-void RTCPInstance::removeSSRC() {
-  fKnownMembers->remove(fLastReceivedSSRC);
+void RTCPInstance::removeLastReceivedSSRC() {
+  removeSSRC(fLastReceivedSSRC);
+}
+
+void RTCPInstance::removeSSRC(u_int32_t ssrc) {
+  fKnownMembers->remove(ssrc);
+
+  // Also, remove records of this SSRC from any reception or transmission stats
+  if (fSource != NULL) fSource->receptionStatsDB().removeRecord(ssrc);
+  if (fSink != NULL) fSink->transmissionStatsDB().removeRecord(ssrc);
 }
 
 void RTCPInstance::onExpire(RTCPInstance* instance) {
@@ -567,7 +572,7 @@ void RTCPInstance::addSR() {
 
   // Insert the NTP and RTP timestamps for the 'wallclock time':
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   fOutBuf->enqueueWord(timeNow.tv_sec + 0x83AA7E80);
       // NTP timestamp most-significant word (1970 epoch -> 1900 epoch)
   double fractionalPart = (timeNow.tv_usec/15625.0)*0x04000000; // 2^32/10^6
@@ -674,7 +679,7 @@ RTCPInstance::enqueueReportBlock(RTPReceptionStats* stats) {
   // Figure out how long has elapsed since the last SR rcvd from this src:
   struct timeval const& LSRtime = stats->lastReceivedSR_time(); // "last SR"
   struct timeval timeNow, timeSinceLSR;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   if (timeNow.tv_usec < LSRtime.tv_usec) {
     timeNow.tv_usec += 1000000;
     timeNow.tv_sec -= 1;
@@ -871,7 +876,7 @@ extern "C" void RemoveMember(packet p) {
   RTCPInstance* instance = (RTCPInstance*)p;
   if (instance == NULL) return;
 
-  instance->removeSSRC();
+  instance->removeLastReceivedSSRC();
 }
 
 extern "C" void RemoveSender(packet /*p*/) {
