@@ -43,12 +43,6 @@ Boolean RTPSink::isRTPSink() const {
   return True;
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 RTPSink::RTPSink(UsageEnvironment& env,
 		 Groupsock* rtpGS, unsigned char rtpPayloadType,
 		 unsigned rtpTimestampFrequency,
@@ -61,18 +55,19 @@ RTPSink::RTPSink(UsageEnvironment& env,
     fNumChannels(numChannels) {
   fRTPPayloadFormatName
     = strDup(rtpPayloadFormatName == NULL ? "???" : rtpPayloadFormatName);
-  gettimeofday(&fCreationTime, &Idunno);
+  gettimeofday(&fCreationTime, NULL);
   fTotalOctetCountStartTime = fCreationTime;
 
-  fSeqNo = (unsigned short)our_random();
-  fSSRC = (unsigned)our_random();
-  fTimestampBase = (unsigned)our_random();
+  fSeqNo = (u_int16_t)our_random();
+  fSSRC = our_random32();
+  fTimestampBase = our_random32();
   fCurrentTimestamp = fTimestampBase;
 
   fTransmissionStatsDB = new RTPTransmissionStatsDB(*this);
 }
 
 RTPSink::~RTPSink() {
+  delete fTransmissionStatsDB;
   delete[] (char*)fRTPPayloadFormatName;
 }
 
@@ -101,15 +96,14 @@ u_int32_t RTPSink::convertToRTPTimestamp(struct timeval tv) const {
 
 u_int32_t RTPSink::timevalToTimestamp(struct timeval tv) const {
   u_int32_t timestamp = (fTimestampFrequency*tv.tv_sec);
-  timestamp += (unsigned)((2.0*fTimestampFrequency*tv.tv_usec + 1000000.0)/2000000);
+  timestamp += (u_int32_t)((2.0*fTimestampFrequency*tv.tv_usec + 1000000.0)/2000000);
        // note: rounding
   return timestamp;
 }
 
-void RTPSink::getTotalBitrate(unsigned& outNumBytes,
-			      double& outElapsedTime) {
+void RTPSink::getTotalBitrate(unsigned& outNumBytes, double& outElapsedTime) {
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
 
   outNumBytes = fTotalOctetCount;
   outElapsedTime = (double)(timeNow.tv_sec-fTotalOctetCountStartTime.tv_sec)
@@ -175,7 +169,7 @@ RTPTransmissionStatsDB::~RTPTransmissionStatsDB() {
 }
 
 void RTPTransmissionStatsDB
-::noteIncomingRR(unsigned SSRC,
+::noteIncomingRR(u_int32_t SSRC, struct sockaddr_in const& lastFromAddress,
                  unsigned lossStats, unsigned lastPacketNumReceived,
                  unsigned jitter, unsigned lastSRTime, unsigned diffSR_RRTime) {
   RTPTransmissionStats* stats = lookup(SSRC);
@@ -190,10 +184,21 @@ void RTPTransmissionStatsDB
 #endif
   }
 
-  stats->noteIncomingRR(lossStats, lastPacketNumReceived, jitter,
+  stats->noteIncomingRR(lastFromAddress,
+			lossStats, lastPacketNumReceived, jitter,
                         lastSRTime, diffSR_RRTime);
 }
 
+void RTPTransmissionStatsDB::removeRecord(u_int32_t SSRC) {
+  RTPTransmissionStats* stats = lookup(SSRC);
+  if (stats != NULL) {
+    long SSRC_long = (long)SSRC;
+    fTable->Remove((char const*)SSRC_long);
+    --fNumReceivers;
+    delete stats;
+  }
+}
+
 RTPTransmissionStatsDB::Iterator
 ::Iterator(RTPTransmissionStatsDB& receptionStatsDB)
   : fIter(HashTable::Iterator::create(*(receptionStatsDB.fTable))) {
@@ -210,12 +215,12 @@ RTPTransmissionStatsDB::Iterator::next() {
   return (RTPTransmissionStats*)(fIter->next(key));
 }
 
-RTPTransmissionStats* RTPTransmissionStatsDB::lookup(unsigned SSRC) const {
+RTPTransmissionStats* RTPTransmissionStatsDB::lookup(u_int32_t SSRC) const {
   long SSRC_long = (long)SSRC;
   return (RTPTransmissionStats*)(fTable->Lookup((char const*)SSRC_long));
 }
 
-void RTPTransmissionStatsDB::add(unsigned SSRC, RTPTransmissionStats* stats) {
+void RTPTransmissionStatsDB::add(u_int32_t SSRC, RTPTransmissionStats* stats) {
   long SSRC_long = (long)SSRC;
   fTable->Add((char const*)SSRC_long, stats);
   ++fNumReceivers;
@@ -224,20 +229,25 @@ void RTPTransmissionStatsDB::add(unsigned SSRC, RTPTransmissionStats* stats) {
 
 ////////// RTPTransmissionStats //////////
 
-RTPTransmissionStats::RTPTransmissionStats(RTPSink& rtpSink, unsigned SSRC)
+RTPTransmissionStats::RTPTransmissionStats(RTPSink& rtpSink, u_int32_t SSRC)
   : fOurRTPSink(rtpSink), fSSRC(SSRC), fLastPacketNumReceived(0),
     fPacketLossRatio(0), fTotNumPacketsLost(0), fJitter(0),
-    fLastSRTime(0), fDiffSR_RRTime(0),
-    fFirstPacket(True) {
+    fLastSRTime(0), fDiffSR_RRTime(0), fFirstPacket(True),
+    fTotalOctetCount_hi(0), fTotalOctetCount_lo(0), 
+    fTotalPacketCount_hi(0), fTotalPacketCount_lo(0) {
+  gettimeofday(&fTimeCreated, NULL);
+
+  fLastOctetCount = rtpSink.octetCount();
+  fLastPacketCount = rtpSink.packetCount();
 }
 
 RTPTransmissionStats::~RTPTransmissionStats() {}
 
-void RTPTransmissionStats::noteIncomingRR(unsigned lossStats,
-					  unsigned lastPacketNumReceived,
-					  unsigned jitter,
-					  unsigned lastSRTime,
-					  unsigned diffSR_RRTime) {
+void RTPTransmissionStats
+::noteIncomingRR(struct sockaddr_in const& lastFromAddress,
+		 unsigned lossStats, unsigned lastPacketNumReceived,
+		 unsigned jitter, unsigned lastSRTime,
+		 unsigned diffSR_RRTime) {
   if (fFirstPacket) {
     fFirstPacket = False;
     fFirstPacketNumReported = lastPacketNumReceived;
@@ -246,8 +256,9 @@ void RTPTransmissionStats::noteIncomingRR(unsigned lossStats,
     fOldLastPacketNumReceived = fLastPacketNumReceived;
     fOldTotNumPacketsLost = fTotNumPacketsLost;
   }
-  gettimeofday(&fTimeReceived, &Idunno);
+  gettimeofday(&fTimeReceived, NULL);
 
+  fLastFromAddress = lastFromAddress;
   fPacketLossRatio = lossStats>>24;
   fTotNumPacketsLost = lossStats&0xFFFFFF;
   fLastPacketNumReceived = lastPacketNumReceived;
@@ -260,6 +271,26 @@ void RTPTransmissionStats::noteIncomingRR(unsigned lossStats,
   unsigned rtd = roundTripDelay(); 
   fprintf(stderr, "=> round-trip delay: 0x%04x (== %f seconds)\n", rtd, rtd/65536.0);
 #endif
+
+  // Update our counts of the total number of octets and packets sent towards
+  // this receiver:
+  u_int32_t newOctetCount = fOurRTPSink.octetCount();
+  u_int32_t octetCountDiff = newOctetCount - fLastOctetCount;
+  fLastOctetCount = newOctetCount;
+  u_int32_t prevTotalOctetCount_lo = fTotalOctetCount_lo;
+  fTotalOctetCount_lo += octetCountDiff;
+  if (fTotalOctetCount_lo < prevTotalOctetCount_lo) { // wrap around
+    ++fTotalOctetCount_hi;
+  }
+
+  u_int32_t newPacketCount = fOurRTPSink.packetCount();
+  u_int32_t packetCountDiff = newPacketCount - fLastPacketCount;
+  fLastPacketCount = newPacketCount;
+  u_int32_t prevTotalPacketCount_lo = fTotalPacketCount_lo;
+  fTotalPacketCount_lo += packetCountDiff;
+  if (fTotalPacketCount_lo < prevTotalPacketCount_lo) { // wrap around
+    ++fTotalPacketCount_hi;
+  }
 }
 
 unsigned RTPTransmissionStats::roundTripDelay() const {
@@ -289,6 +320,16 @@ unsigned RTPTransmissionStats::roundTripDelay() const {
   return (unsigned)rawResult;
 }
 
+void RTPTransmissionStats::getTotalOctetCount(u_int32_t& hi, u_int32_t& lo) {
+  hi = fTotalOctetCount_hi;
+  lo = fTotalOctetCount_lo;
+}
+
+void RTPTransmissionStats::getTotalPacketCount(u_int32_t& hi, u_int32_t& lo) {
+  hi = fTotalPacketCount_hi;
+  lo = fTotalPacketCount_lo;
+}
+
 unsigned RTPTransmissionStats::packetsReceivedSinceLastRR() const {
   if (!fOldValid) return 0;
 
