@@ -613,7 +613,6 @@ Boolean RTSPClient::announceSDPDescription(char const* url,
       envir().setResultMsg("cannot handle ANNOUNCE response: ", firstLine);
       break;
     }
-    // (Later, check "CSeq" too #####)
 
     delete[] cmd;
     return True;
@@ -689,16 +688,35 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
       if (subsession.parentSession().isRealNetworksRDT) {
 	transportHeader = strDup("Transport: x-pn-tng/tcp;mode=play,rtp/avp/unicast;mode=play\r\n");
       } else {
-	// Use a regular "Transport:" header, but ask for TCP streaming:
-	streamUsingTCP = True;
+	// Use a regular "Transport:" header:
 	char const* transportHeaderFmt
-	  = "Transport: RTP/AVP/TCP;unicast;interleaved=%d-%d\r\n";
+	  = "Transport: RTP/AVP%s%s=%d-%d\r\n";
+	char const* transportTypeStr;
+	char const* portTypeStr;
+	unsigned short rtpNumber, rtcpNumber;
+	if (streamUsingTCP) { // streaming over the RTSP connection
+	  transportTypeStr = "/TCP;unicast";
+	  portTypeStr = ";interleaved";
+	  rtpNumber = fTCPStreamIdCount++;
+	  rtcpNumber = fTCPStreamIdCount++;
+	} else { // normal RTP streaming
+	  unsigned connectionAddress = subsession.connectionEndpointAddress();
+	  transportTypeStr
+	    = IsMulticastAddress(connectionAddress) ? ";multicast" : ";unicast";
+	  portTypeStr = ";client_port";
+	  rtpNumber = subsession.clientPortNum();
+	  if (rtpNumber == 0) {
+	    envir().setResultMsg("Client port number unknown\n");
+	    break;
+	  }
+	  rtcpNumber = rtpNumber + 1;
+	}
+
 	unsigned transportHeaderSize = strlen(transportHeaderFmt)
-	  + 2*5 /* max port len */;
+	  + strlen(transportTypeStr) + strlen(portTypeStr) + 2*5 /* max port len */;
 	transportHeader = new char[transportHeaderSize];
-	unsigned short rtpNumber = fTCPStreamIdCount++;
-	unsigned short rtcpNumber = fTCPStreamIdCount++;
-	sprintf(transportHeader, transportHeaderFmt, rtpNumber, rtcpNumber);
+	sprintf(transportHeader, transportHeaderFmt,
+		transportTypeStr, portTypeStr, rtpNumber, rtcpNumber);
       }
       char const* transportFmt =
 	"%s"
@@ -728,9 +746,7 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
     constructSubsessionURL(subsession, prefix, separator, suffix);
     char* transportFmt;
 
-    if (fServerIsKasenna && fKasennaContentType != NULL &&
-	(strncmp(fKasennaContentType, "MPEG-2", 6) == 0 ||
-	 strncmp(fKasennaContentType, "MPEG-1", 6) == 0)) {
+    if (strcmp(subsession.protocolName(), "UDP") == 0) {
       char const* setupFmt = "SETUP %s%s RTSP/1.0\r\n";
       unsigned setupSize = strlen(setupFmt)
         + strlen(prefix) + strlen (separator);
@@ -819,7 +835,7 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
 
     // Look for a "Session:" header (to set our session id), and
     // a "Transport: " header (to set the server address/port)
-    // For now, ignore other headers.  (Later, check for "CSeq" also #####)
+    // For now, ignore other headers.
     char* lineStart;
     char* sessionId = new char[fResponseBufferSize]; // ensures we have enough space
     while (1) {
@@ -879,6 +895,18 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
   return False;
 }
 
+static char* createScaleString(float scale, float currentScale) {
+  char buf[100];
+  if (scale == 1.0f && currentScale == 1.0f) {
+    // This is the default value; we don't need a "Scale:" header:
+    buf[0] = '\0';
+  } else {
+    sprintf(buf, "Scale: %f\r\n", scale);
+  }
+
+  return strDup(buf);
+}
+      
 static char* createRangeString(float start, float end) {
   char buf[100];
   if (start < 0) {
@@ -896,7 +924,7 @@ static char* createRangeString(float start, float end) {
 }
       
 Boolean RTSPClient::playMediaSession(MediaSession& session,
-				     float start, float end) {
+				     float start, float end, float scale) {
 #ifdef SUPPORT_REAL_RTSP
   if (session.isRealNetworksRDT) {
     // This is a RealNetworks stream; set the "Subscribe" parameter before proceeding:
@@ -918,6 +946,8 @@ Boolean RTSPClient::playMediaSession(MediaSession& session,
     // First, construct an authenticator string:
     char* authenticatorStr
       = createAuthenticatorString(&fCurrentAuthenticator, "PLAY", fBaseURL);
+    // And then a "Scale:" string:
+    char* scaleStr = createScaleString(scale, session.scale());
     // And then a "Range:" string:
     char* rangeStr = createRangeString(start, end);
 
@@ -928,12 +958,14 @@ Boolean RTSPClient::playMediaSession(MediaSession& session,
       "%s"
       "%s"
       "%s"
+      "%s"
       "\r\n";
 
     unsigned cmdSize = strlen(cmdFmt)
       + strlen(fBaseURL)
       + 20 /* max int len */
       + strlen(fLastSessionId)
+      + strlen(scaleStr)
       + strlen(rangeStr)
       + strlen(authenticatorStr)
       + fUserAgentHeaderStrSize;
@@ -942,9 +974,11 @@ Boolean RTSPClient::playMediaSession(MediaSession& session,
 	    fBaseURL,
 	    ++fCSeq,
 	    fLastSessionId,
+	    scaleStr,
 	    rangeStr,
 	    authenticatorStr,
 	    fUserAgentHeaderStr);
+    delete[] scaleStr;
     delete[] rangeStr;
     delete[] authenticatorStr;
 
@@ -955,6 +989,17 @@ Boolean RTSPClient::playMediaSession(MediaSession& session,
     char* firstLine; char* nextLineStart;
     if (!getResponse("PLAY", bytesRead, responseCode, firstLine, nextLineStart)) break;
 
+    // Look for various headers that we understand:
+    char* lineStart;
+    while (1) {
+      lineStart = nextLineStart;
+      if (lineStart == NULL) break;
+
+      nextLineStart = getLine(lineStart);
+
+      if (parseScaleHeader(lineStart, session.scale())) break;
+    }
+
     delete[] cmd;
     return True;
   } while (0);
@@ -964,7 +1009,7 @@ Boolean RTSPClient::playMediaSession(MediaSession& session,
 }
 
 Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
-					float start, float end,
+					float start, float end, float scale,
 					Boolean hackForDSS) {
   char* cmd = NULL;
   do {
@@ -979,6 +1024,8 @@ Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
     // First, construct an authenticator string:
     char* authenticatorStr
       = createAuthenticatorString(&fCurrentAuthenticator, "PLAY", fBaseURL);
+    // And then a "Scale:" string:
+    char* scaleStr = createScaleString(scale, subsession.scale());
     // And then a "Range:" string:
     char* rangeStr = createRangeString(start, end);
 
@@ -989,6 +1036,7 @@ Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
       "%s"
       "%s"
       "%s"
+      "%s"
       "\r\n";
 
     char const *prefix, *separator, *suffix;
@@ -1005,6 +1053,7 @@ Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
       + strlen(prefix) + strlen(separator) + strlen(suffix)
       + 20 /* max int len */
       + strlen(subsession.sessionId)
+      + strlen(scaleStr)
       + strlen(rangeStr)
       + strlen(authenticatorStr)
       + fUserAgentHeaderStrSize;
@@ -1013,9 +1062,11 @@ Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
 	    prefix, separator, suffix,
 	    ++fCSeq,
 	    subsession.sessionId,
+	    scaleStr,
 	    rangeStr,
 	    authenticatorStr,
 	    fUserAgentHeaderStr);
+    delete[] scaleStr;
     delete[] rangeStr;
     delete[] authenticatorStr;
 
@@ -1026,7 +1077,7 @@ Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
     char* firstLine; char* nextLineStart;
     if (!getResponse("PLAY", bytesRead, responseCode, firstLine, nextLineStart)) break;
 
-    // Look for a "RTP-Info:" header:
+    // Look for various headers that we understand:
     char* lineStart;
     while (1) {
       lineStart = nextLineStart;
@@ -1038,12 +1089,11 @@ Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
 			     subsession.rtpInfo.trackId,
 			     subsession.rtpInfo.seqNum,
 			     subsession.rtpInfo.timestamp)) {
-	break;
+	continue;
       }
+      if (parseScaleHeader(lineStart, subsession.scale())) continue;
     }
 
-    // (Later, check "CSeq" too #####)
-
     delete[] cmd;
     return True;
   } while (0);
@@ -1405,12 +1455,6 @@ Boolean RTSPClient::teardownMediaSubsession(MediaSubsession& subsession) {
   return False;
 }
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
-
 Boolean RTSPClient::openConnectionFromURL(char const* url) {
   do {
     // Set this as our base URL:
@@ -1909,6 +1953,13 @@ Boolean RTSPClient::parseRTPInfoHeader(char const* line,
   return True;
 }
 
+Boolean RTSPClient::parseScaleHeader(char const* line, float& scale) {
+  if (_strncasecmp(line, "Scale: ", 7) != 0) return False;
+  line += 7;
+
+  return sscanf(line, "%f", &scale) == 1;
+}
+
 Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix) {
   if (fVerbosityLevel >= 1) {
     envir() << "Requesting RTSP-over-HTTP tunneling (on port "
@@ -1923,7 +1974,7 @@ Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix) {
       struct timeval timestamp;
       unsigned counter;
     } seedData;
-    gettimeofday(&seedData.timestamp, &Idunno);
+    gettimeofday(&seedData.timestamp, NULL);
     static unsigned counter = 0;
     seedData.counter = ++counter;
     char sessionCookie[33];
