@@ -59,6 +59,8 @@ Boolean createReceivers = True;
 Boolean outputQuickTimeFile = False;
 Boolean generateMP4Format = False;
 QuickTimeFileSink* qtOut = NULL;
+Boolean outputAVIFile = False;
+AVIFileSink* aviOut = NULL;
 Boolean audioOnly = False;
 Boolean videoOnly = False;
 char const* singleMedium = NULL;
@@ -81,9 +83,12 @@ char* proxyServerName = NULL;
 unsigned short proxyServerPortNum = 0;
 unsigned char desiredAudioRTPPayloadFormat = 0;
 char* mimeSubtype = NULL;
-unsigned short movieWidth = 240;
-unsigned short movieHeight = 180;
-unsigned movieFPS = 15;
+unsigned short movieWidth = 240; // default
+Boolean movieWidthOptionSet = False;
+unsigned short movieHeight = 180; // default
+Boolean movieHeightOptionSet = False;
+unsigned movieFPS = 15; // default
+Boolean movieFPSOptionSet = False;
 char* fileNamePrefix = "";
 unsigned fileSinkBufferSize = 20000;
 unsigned socketInputBufferSize = 0;
@@ -94,21 +99,16 @@ char* destRTSPURL = NULL;
 unsigned qosMeasurementIntervalMS = 0; // 0 means: Don't output QOS data
 unsigned statusCode = 0;
 
-#ifdef BSD
-static struct timezone Idunno;
-#else
-static int Idunno;
-#endif
 struct timeval startTime;
 
 void usage() {
   *env << "Usage: " << progName
-       << " [-p <startPortNum>] [-r|-q|-4] [-a|-v] [-V] [-e <endTime>] [-E <max-inter-packet-gap-time> [-c] [-s <offset>] [-n] [-O]"
+       << " [-p <startPortNum>] [-r|-q|-4|-i] [-a|-v] [-V] [-e <endTime>] [-E <max-inter-packet-gap-time> [-c] [-s <offset>] [-n] [-O]"
 	   << (controlConnectionUsesTCP ? " [-t|-T <http-port>]" : "")
        << " [-u <username> <password>"
 	   << (allowProxyServers ? " [<proxy-server> [<proxy-server-port>]]" : "")
        << "]" << (supportCodecSelection ? " [-A <audio-codec-rtp-payload-format-code>|-D <mime-subtype-name>]" : "")
-       << " [-w <width> -h <height>] [-f <frames-per-second>] [-y] [-H] [-Q [<measurement-interval>]] [-F <filename-prefix>] [-b <file-sink-buffer-size>] [-B <input-socket-buffer-size>] [-m] <url> (or " << progName << " -o [-V] <url>)\n";
+       << " [-w <width> -h <height>] [-f <frames-per-second>] [-y] [-H] [-Q [<measurement-interval>]] [-F <filename-prefix>] [-b <file-sink-buffer-size>] [-B <input-socket-buffer-size>] [-I <input-interface-ip-address>] [-m] <url> (or " << progName << " -o [-V] <url>)\n";
   //##### Add "-R <dest-rtsp-url>" #####
   shutdown();
 }
@@ -120,7 +120,7 @@ int main(int argc, char** argv) {
 
   progName = argv[0];
 
-  gettimeofday(&startTime, &Idunno);
+  gettimeofday(&startTime, NULL);
 
 #ifdef USE_SIGNALS
   // Allow ourselves to be shut down gracefully by a SIGHUP or a SIGUSR1:
@@ -167,6 +167,22 @@ int main(int argc, char** argv) {
       break;
     }
 
+    case 'i': { // output an AVI file (to stdout)
+      outputAVIFile = True;
+      break;
+    }
+
+    case 'I': { // specify input interface... 
+      NetAddressList addresses(argv[2]);
+      if (addresses.numAddresses() == 0) {
+	*env << "Failed to find network address for \"" << argv[2] << "\"";
+	break;
+      }
+      ReceivingInterfaceAddr = *(unsigned*)(addresses.firstAddress()->data());
+      ++argv; --argc;
+      break;
+    }
+
     case 'a': { // receive/record an audio stream only
       audioOnly = True;
       singleMedium = "audio";
@@ -312,26 +328,29 @@ int main(int argc, char** argv) {
       break;
     }
 
-    case 'w': { // specify a width (pixels) for an output QuickTime movie
+    case 'w': { // specify a width (pixels) for an output QuickTime or AVI movie
       if (sscanf(argv[2], "%hu", &movieWidth) != 1) {
 	usage();
       }
+      movieWidthOptionSet = True;
       ++argv; --argc;
       break;
     }
 
-    case 'h': { // specify a height (pixels) for an output QuickTime movie
+    case 'h': { // specify a height (pixels) for an output QuickTime or AVI movie
       if (sscanf(argv[2], "%hu", &movieHeight) != 1) {
 	usage();
       }
+      movieHeightOptionSet = True;
       ++argv; --argc;
       break;
     }
 
-    case 'f': { // specify a frame rate (per second) for an output QT movie
+    case 'f': { // specify a frame rate (per second) for an output QT or AVI movie
       if (sscanf(argv[2], "%u", &movieFPS) != 1) {
 	usage();
       }
+      movieFPSOptionSet = True;
       ++argv; --argc;
       break;
     }
@@ -404,14 +423,31 @@ int main(int argc, char** argv) {
     ++argv; --argc;
   }
   if (argc != 2) usage();
-  if (!createReceivers && outputQuickTimeFile) {
-    *env << "The -r and -q (or -4) flags cannot both be used!\n";
+  if (outputQuickTimeFile && outputAVIFile) {
+    *env << "The -i and -q (or -4) flags cannot both be used!\n";
     usage();
   }
-  if (destRTSPURL != NULL && (!createReceivers || outputQuickTimeFile)) {
-    *env << "The -R flag cannot be used with -r or -q!\n";
+  Boolean outputCompositeFile = outputQuickTimeFile || outputAVIFile;
+  if (!createReceivers && outputCompositeFile) {
+    *env << "The -r and -q (or -4 or -i) flags cannot both be used!\n";
     usage();
   }
+  if (destRTSPURL != NULL && (!createReceivers || outputCompositeFile)) {
+    *env << "The -R flag cannot be used with -r, -q, or -i!\n";
+    usage();
+  }
+  if (outputCompositeFile && !movieWidthOptionSet) {
+    *env << "Warning: The -q, -4 or -i option was used, but not -w.  Assuming a video width of "
+	 << movieWidth << " pixels\n";
+  }
+  if (outputCompositeFile && !movieHeightOptionSet) {
+    *env << "Warning: The -q, -4 or -i option was used, but not -h.  Assuming a video height of "
+	 << movieHeight << " pixels\n";
+  }
+  if (outputCompositeFile && !movieFPSOptionSet) {
+    *env << "Warning: The -q, -4 or -i option was used, but not -f.  Assuming a video frame rate of "
+	 << movieFPS << " frames-per-second\n";
+  }
   if (audioOnly && videoOnly) {
     *env << "The -a and -v flags cannot both be used!\n";
     usage();
@@ -580,11 +616,24 @@ int main(int argc, char** argv) {
 					   generateHintTracks,
 					   generateMP4Format);
       if (qtOut == NULL) {
-		*env << "Failed to create QuickTime file sink for stdout: " << env->getResultMsg();
-		shutdown();
+	*env << "Failed to create QuickTime file sink for stdout: " << env->getResultMsg();
+	shutdown();
       }
 
       qtOut->startPlaying(sessionAfterPlaying, NULL);
+    } else if (outputAVIFile) {
+      // Create an "AVIFileSink", to write to 'stdout':
+      aviOut = AVIFileSink::createNew(*env, *session, "stdout",
+				      fileSinkBufferSize,
+				      movieWidth, movieHeight,
+				      movieFPS,
+				      packetLossCompensate);
+      if (aviOut == NULL) {
+	*env << "Failed to create AVI file sink for stdout: " << env->getResultMsg();
+	shutdown();
+      }
+
+      aviOut->startPlaying(sessionAfterPlaying, NULL);
 #ifdef SUPPORT_REAL_RTSP
     } else if (session->isRealNetworksRDT) {
       // For RealNetworks' sessions, we create a single output file,
@@ -604,7 +653,7 @@ int main(int argc, char** argv) {
       unsigned headerSize;
       unsigned char* headerData = RealGenerateRMFFHeader(session, headerSize);
       struct timeval timeNow;
-      gettimeofday(&timeNow, &Idunno);
+      gettimeofday(&timeNow, NULL);
       fileSink->addData(headerData, headerSize, timeNow);
       delete[] headerData;
 
@@ -689,7 +738,7 @@ int main(int argc, char** argv) {
 	    unsigned char* configData
 	      = parseGeneralConfigStr(subsession->fmtp_config(), configLen);
 	    struct timeval timeNow;
-	    gettimeofday(&timeNow, &Idunno);
+	    gettimeofday(&timeNow, NULL);
 	    fileSink->addData(configData, configLen, timeNow);
 	    delete[] configData;
 	  }
@@ -812,6 +861,7 @@ void tearDownStreams() {
 
 void closeMediaSinks() {
   Medium::close(qtOut);
+  Medium::close(aviOut);
 
   if (session == NULL) return;
   MediaSubsessionIterator iter(*session);
@@ -841,7 +891,7 @@ void subsessionAfterPlaying(void* clientData) {
 
 void subsessionByeHandler(void* clientData) {
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   unsigned secsDiff = timeNow.tv_sec - startTime.tv_sec;
 
   MediaSubsession* subsession = (MediaSubsession*)clientData;
@@ -922,7 +972,7 @@ static unsigned nextQOSMeasurementUSecs;
 static void scheduleNextQOSMeasurement() {
   nextQOSMeasurementUSecs += qosMeasurementIntervalMS*1000;
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
   unsigned timeNowUSecs = timeNow.tv_sec*1000000 + timeNow.tv_usec;
   unsigned usecsToDelay = nextQOSMeasurementUSecs < timeNowUSecs ? 0
     : nextQOSMeasurementUSecs - timeNowUSecs;
@@ -933,7 +983,7 @@ static void scheduleNextQOSMeasurement() {
 
 static void periodicQOSMeasurement(void* /*clientData*/) {
   struct timeval timeNow;
-  gettimeofday(&timeNow, &Idunno);
+  gettimeofday(&timeNow, NULL);
 
   for (qosMeasurementRecord* qosRecord = qosRecordHead;
        qosRecord != NULL; qosRecord = qosRecord->fNext) {
@@ -1004,7 +1054,7 @@ void qosMeasurementRecord
 void beginQOSMeasurement() {
   // Set up a measurement record for each active subsession:
   struct timeval startTime;
-  gettimeofday(&startTime, &Idunno);
+  gettimeofday(&startTime, NULL);
   nextQOSMeasurementUSecs = startTime.tv_sec*1000000 + startTime.tv_usec;
   qosMeasurementRecord* qosRecordTail = NULL;
   MediaSubsessionIterator iter(*session);
@@ -1183,10 +1233,12 @@ void checkForPacketArrival(void* /*clientData*/) {
   }
 
   unsigned numSubsessionsToCheck = numSubsessionsChecked;
+  // Special case for "QuickTimeFileSink"s and "AVIFileSink"s:
+  // They might not use all of the input sources:
   if (qtOut != NULL) {
-    // Special case for "QuickTimeFileSink"s: They might not use all of the
-    // input sources:
     numSubsessionsToCheck = qtOut->numActiveSubsessions();
+  } else if (aviOut != NULL) {
+    numSubsessionsToCheck = aviOut->numActiveSubsessions();
   }
 
   Boolean notifyTheUser;
@@ -1199,7 +1251,7 @@ void checkForPacketArrival(void* /*clientData*/) {
   }
   if (notifyTheUser) {
     struct timeval timeNow;
-    gettimeofday(&timeNow, &Idunno);
+    gettimeofday(&timeNow, NULL);
 	char timestampStr[100];
 	sprintf(timestampStr, "%ld%03ld", timeNow.tv_sec, timeNow.tv_usec/1000);
     *env << (syncStreams ? "Synchronized d" : "D")
@@ -1392,7 +1444,7 @@ Boolean setupDestinationRTSPServer() {
     if (!rtspClientOutgoing->setupMediaSubsession(*destSubsession,
 						  True, True)) break;
     if (!rtspClientOutgoing->playMediaSubsession(*destSubsession,
-						 0.0, -1.0,
+						 0.0, -1.0, 1.0,
 						 True/*hackForDSS*/)) break;
 
     // Next, set up "RTPSink"s for the outgoing packets:
